/*! \file urbi/uobject.hh
 *******************************************************************************

 File: uobject.hh\n
 Definition of the UObject class and necessary related classes.

 This file is part of UObject Component Architecture\n
 (c) 2006 Gostai S.A.S.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://www.urbiforge.com

 **************************************************************************** */

#ifndef URBI_UOBJECT_HH
# define URBI_UOBJECT_HH

# include <string>
# include <list>
# include <algorithm>

# include "libport/singleton-ptr.hh"
# include "libport/ufloat.h"
# include "libport/hash.hh"

# include "urbi/fwd.hh"
# include "urbi/ubinary.hh"
# include "urbi/utypes-common.hh"
# include "urbi/uvalue.hh"
# include "urbi/uvar.hh"

/// This macro must be called once for every UObject class.
# define UStart(x)							\
  ::urbi::URBIStarter<x> x ##  ____URBI_object(std::string(# x),	\
					       ::urbi::objectlist)

/// This macro must be called once for every UObject class.
# define UStartRename(x,name)						\
  ::urbi::URBIStarter<x> x ##  ____URBI_object(std::string(# name),	\
					       ::urbi::objectlist)

/// This macro must be called once for each UObjectHub class.
# define UStartHub(x)							\
  ::urbi::URBIStarterHub<x> x ##  ____URBI_object(std::string(# x),	\
						  ::urbi::objecthublist)


/** Bind a variable to an object.

 This macro can only be called from within a class inheriting from
 UObject.  It binds the UVar x within the object to a variable
 with the same name in the corresponding URBI object.  */
# define UBindVar(obj,x) x.init(__name,# x)

/** This macro inverts a UVar in/out accesses.

 After this call is made, writes by this module affect the sensed
 value, and reads read the target value. Writes by other modules
 and URBI code affect the target value, and reads get the sensed
 value. Without this call, all operations affect the same
 underlying variable.  */
# define UOwned(x) x.setOwned()
// for backward compatibility
# define USensor(x) x.setOwned()

/** Bind the function x in current URBI object to the C++ member
 function of same name.  The return value and parameters must be of
 a basic integral or floating types, char *, std::string, UValue,
 UBinary, USound or UImage, or any type that can cast to/from
 UValue.  */
# define UBindFunction(obj,x)						\
  ::urbi::createUCallback(__name, "function", this,			\
			  (&obj::x),__name+"."+std::string(# x),	\
			  ::urbi::functionmap)

/** Registers a function x in current object that will be called each
 time the event of same name is triggered. The function will be
 called only if the number of arguments match between the function
 prototype and the URBI event.
 */
# define UBindEvent(obj,x)						\
  ::urbi::createUCallback(__name, "event", this,			\
			  (&obj::x),__name+"."+std::string(# x),	\
			  ::urbi::eventmap)

/** Registers a function x in current object that will be called each
 * time the event of same name is triggered, and a function fun called
 * when the event ends. The function will be called only if the number
 * of arguments match between the function prototype and the URBI
 * event.
 */
# define UBindEventEnd(obj,x,fun)					\
  ::urbi::createUCallback(__name, "eventend", this,			\
			  (&obj::x),(&obj::fun),__name+"."+std::string(# x), \
			  ::urbi::eventendmap)

/// Register current object to the UObjectHub named 'hub'.
# define URegister(hub)						\
  do {								\
    objecthub = ::urbi::getUObjectHub((std::string) #hub);	\
    if (objecthub)						\
      objecthub->addMember(dynamic_cast<UObject*>(this));	\
    else							\
      ::urbi::echo("Error: hub name '%s' is unknown\n", #hub);	\
  } while (0)


//macro to send urbi commands
# ifndef URBI
/** Send unquoted URBI commands to the server. Add an extra layer of parenthesis
    for safety.
*/
#   define URBI(a) ::urbi::uobject_unarmorAndSend(# a)
# endif


namespace urbi
{

  // A few list and hashtable types
  typedef libport::hash_map_type<std::string, std::list<UGenericCallback*> >::type UTable;
  typedef libport::hash_map_type<std::string, std::list<UVar*> >::type UVarTable;
  typedef std::list<baseURBIStarter*> UStartlist;
  typedef std::list<baseURBIStarterHub*> UStartlistHub;
  typedef std::list<UTimerCallback*> UTimerTable;
  typedef std::list<UObject*> UObjectList;

  typedef int UReturn;

  // Two singleton lists to handle the object and hubobject registration.
  EXTERN_STATIC_INSTANCE(UStartlist, objectlist);
  EXTERN_STATIC_INSTANCE(UStartlistHub, objecthublist);

  // Lists and hashtables used.
  extern UVarTable varmap;
  extern UTable functionmap;
  extern UTable eventmap;
  extern UTable eventendmap;
  extern UTable monitormap;
  extern UTable accessmap;

  // Timer and update maps.
  extern UTimerTable timermap;
  extern UTimerTable updatemap;

  // For remote mode.
  extern void main(int argc, char *argv[]);

  /// an empty dummy UObject used by UVar to set a NotifyChange
  /// This avoid coupling a UVar to a particular object
  extern UObject* dummyUObject;

  // Global function of the urbi:: namespace to access kernel features

  /// Write a message to the server debug output. Printf syntax.
  void echo(const char * format, ... );
  /// Retrieve a UObjectHub based on its name or return 0 if not found.
  UObjectHub* getUObjectHub(const std::string& n);
  /// Retrieve a UObject based on its name or return 0 if not found.
  UObject* getUObject(const std::string& n);

  /// Send URBI code (ghost connection in plugin mode, default
  /// connection in remote mode).
  void uobject_unarmorAndSend(const char * str);
  /// Send the string to the connection hosting the UObject.
  void send(const char* str);
  /// Send buf to the connection hosting the UObject.
  void send(void* buf,int size);

  //! Function and Event storage mechanism
  /*! This heavily overloaded class is the only way in C++ to make life easy from the
    the interface user point's of view.
  */

  class UGenericCallback
  {
  public:
    UGenericCallback(const std::string& objname,
		     const std::string& type,
		     const std::string& name,
		     int size, UTable &t);
    UGenericCallback(const std::string& objname,
		     const std::string& type,
		     const std::string& name, UTable &t);
    virtual ~UGenericCallback();
    std::string getName() { return name;};

    virtual UValue __evalcall(UList &param)  = 0;

    /// Used to store the UVar* pointeur for var monitoring.
    void   *storage;
    /// Period of timers.
    ufloat period;
    /// Nb params of the callbacked function.
    int    nbparam;
    /// Name of the UObject that has created the callback.
    std::string objname;

  private:
    std::string name;
  };


  //! Timer mechanism
  /*! This class stores a callback as a class method
   */

  class UTimerCallback
  {
  public:
    UTimerCallback(const std::string& objname, ufloat period, UTimerTable &tt);
    virtual ~UTimerCallback();

    virtual void call() = 0;

    ufloat period;
    ufloat lastTimeCalled;
    std::string objname;
  };

  // UTimerCallback subclasses

  template <class T>
  class UTimerCallbackobj : public UTimerCallback
  {
  public:
    UTimerCallbackobj(const std::string& objname,
		      ufloat period, T* obj, int (T::*fun) (), UTimerTable &tt)
      : UTimerCallback(objname, period,tt), obj(obj), fun(fun)
    {}

    virtual void call()
    {
      ((*obj).*fun)();
    }
  private:
    T* obj;
    int (T::*fun) ();
  };


  /** Main UObject class definition
      Each UObject instance corresponds to an URBI object. It provides mechanisms to
      bind variables and functions between C++ and URBI.
  */
  class UObject
  {
  public:

    UObject(const std::string&);
    /// dummy UObject constructor
    UObject(int);
    virtual ~UObject();

    /// Calls the specified function each time the variable v is modified.
    template <class T>
    void UNotifyChange (UVar& v, int (T::*fun) ())
    {
      createUCallback(__name, "var", (T*)this, fun, v.get_name(), monitormap);
    }

    /// Calls the specified function each time the variable v is modified.
    template <class T>
    void UNotifyChange (UVar& v, int (T::*fun) (UVar&))
    {
      UGenericCallback* cb =
	createUCallback(__name, "var", (T*)this, fun, v.get_name(), monitormap);
      if (cb)
	cb->storage = &v;
    }

    /// Calls the specified function when the variable value is updated on request by requestValue().
    template <class T>
    void UNotifyOnRequest (UVar& v, int (T::*fun) ())
    {
      createUCallback(__name, "var_onrequest", (T*)this,
		      fun, v.get_name(), monitormap);
    }

    /// Calls the specified function when the variable value is updated on request by requestValue().
    template <class T>
    void UNotifyOnRequest (UVar& v, int (T::*fun) (UVar&))
    {
      UGenericCallback* cb =
	createUCallback(__name, "var_onrequest", (T*)this,
			fun, v.get_name(), monitormap);
      if (cb)
	cb->storage = &v;
    }

    /// Calls the specified function each time the variable v is modified.
    template <class T>
    void UNotifyChange (const std::string& name, int (T::*fun) ())
    {
      createUCallback(__name, "var", (T*)this, fun, name, monitormap);
    }

    /// Calls the specified function each time the variable v is modified.
    template <class T>
    void UNotifyChange (const std::string& name, int (T::*fun) (UVar&))
    {
      UGenericCallback* cb =
	createUCallback(__name, "var", (T*)this, fun, name, monitormap);
      if (cb)
	cb->storage = new UVar(name);
    }

    /// Calls the specified function each time the variable v is read.
    template <class T>
    void UNotifyAccess (UVar& v, int (T::*fun) (UVar&))
    {
      UGenericCallback* cb =
	createUCallback(__name, "varaccess", (T*)this,
			fun, v.get_name(), accessmap);
      if (cb)
	cb->storage = &v;
    }

    /// Set a timer that will call tune 'fun' function every 't' milliseconds.
    template <class T>
    void USetTimer(ufloat t, int (T::*fun) ())
    {
      new UTimerCallbackobj<T> (__name, t,(T*)this, fun, timermap);
    }

    /// Request permanent synchronization for v.
    void USync(UVar &v);

    /// Name of the object as seen in URBI.
    std::string __name;
    /// Name of the class the object is derived from.
    std::string classname;
    /// True when the object has been newed by an urbi command.
    bool   derived;

    UObjectList members;
    /// The hub, if it exists.
    UObjectHub  *objecthub;

    /// Set a timer that will call the update function every 'period'
    /// milliseconds
    void USetUpdate(ufloat period);
    virtual int update() {return 0;};
    /// Set autogrouping facility for each new subclass created..
    void UAutoGroup() { autogroup = true; };
    /// Called when a subclass is created if autogroup is true.
    virtual void addAutoGroup() { UJoinGroup(classname+"s"); };

    /// Join the uobject to the 'gpname' group.
    virtual void UJoinGroup(const std::string& gpname);
    /// Void function used in USync callbacks.
    int voidfun() {return  0;};
    /// Add a group with a 's' after the base class name.
    bool autogroup;

    /// Flag to know whether the UObject is in remote mode or not
    bool remote;

    /// Remove all bindings, this method is called by the destructor.
    void clean();

    /// The load attribute is standard and can be used to control the
    /// activity of the object.
    UVar load;

  private:
    /// Pointer to a globalData structure specific to the
    /// remote/plugin architectures who defines it.
    UObjectData*  objectData;
    ufloat period;
  };



  //! Main UObjectHub class definition
  class UObjectHub
  {
  public:

    UObjectHub(const std::string&);
    virtual ~UObjectHub();

    void addMember(UObject* obj);

    /// Set a timer that will call update every 'period' milliseconds.
    void USetUpdate(ufloat period);
    virtual int update() {return 0;}

    UObjectList  members;
    UObjectList* getSubClass(const std::string&);
    //   UObjectList* getAllSubClass(const std::string&); //TODO

  protected:
    /// This function calls update and the subclass update.
    int updateGlobal();

    ufloat period;
    std::string name;
  };


  /*----------.
  | Casters.  |
  `----------*/

# define SETCAST(Type)				\
  inline Type					\
  cast(UValue &val, Type*)			\
  {						\
    return (Type)val;				\
  }

  inline const UValue & cast(UValue &val, const UValue *)
  {
    return val;
  }

  inline UValue cast(UValue &val,  UValue *)
  {
    return val;
  }
  SETCAST(int);
  SETCAST(unsigned int);
  SETCAST(long);
  SETCAST(unsigned long);
  SETCAST(ufloat);
  SETCAST(std::string);
  SETCAST(const std::string);
  SETCAST(bool);
  SETCAST(UImage);
  SETCAST(USound);
  UVar& cast(UValue &val, UVar *var);
  UBinary cast(UValue &v, UBinary * b);
  UList cast(UValue &v, UList *l);
  UObjectStruct cast(UValue &v, UObjectStruct *o);
  const char * cast(UValue &v, const char ** b);

#ifndef UOBJECT_NO_LIST_CAST
template<class I> std::list<I> cast(UValue &val, std::list<I> * inu)
{
  std::list<I> result;
  if (val.type != DATA_LIST)
    //cast just the element
    result.push_back(cast(val, (I*)0));
  else
    for (int i=0;i<val.list->size();i++)
      result.push_back(cast(*val.list->array[i], (I*)0));
  return result;
}
#endif

  /// URBIStarter base class used to store heterogeneous template
  /// class objects in starterlist.
  class baseURBIStarter
  {
  public:
    baseURBIStarter(const std::string& name)
      : name(name)
    {}
    virtual ~baseURBIStarter() {}

    virtual UObject* getUObject() = 0;

    /// Called before deletion.
    virtual void clean() = 0;
    /// Used to provide a wrapper to initialize objects in starterlist.
    virtual void init(const std::string&) = 0;
    /// Used to provide a copy of a C++ object based on its name.
    virtual void copy(const std::string&) = 0;
    std::string name;
  };

  //! This is the class containing URBI starters
  /** A starter is a class whose job is to start an instance of a particular UObject subclass,
   * resulting in the initialization of this object (registration to the kernel)
   */
  template <class T>
  class URBIStarter
    : public baseURBIStarter
  {
  public:
    URBIStarter(const std::string& name, UStartlist& _slist)
      : baseURBIStarter(name)
    {
      slist = &_slist;
      slist->push_back(dynamic_cast<baseURBIStarter*>(this));
    }

    virtual ~URBIStarter()
    {
      clean();
    }

    virtual void clean()
    {
      delete getUObject();
      UStartlist::iterator toerase =
	std::find(slist->begin(), slist->end(),
		  dynamic_cast<baseURBIStarter*>(this));
      if (toerase != slist->end())
	slist->erase(toerase);
    }

    virtual void
    copy(const std::string& objname)
    {
      URBIStarter<T>* ustarter = new URBIStarter<T>(objname,*slist);
      ustarter->init(objname);
      UObject *uso = dynamic_cast<UObject*>(ustarter->object);
      getUObject()->members.push_back(uso);
      uso->derived   = true;
      uso->classname = getUObject()->classname;
      if (uso->autogroup)
	uso->addAutoGroup();
    }

    /// Access to the object from the outside.
    virtual UObject* getUObject()
    {
      return dynamic_cast<UObject*>(object);
    }

  protected:
    /// Called when the object is ready to be initialized.
    virtual void init(const std::string& objname)
    {
      object = new T(objname);
    }

    UStartlist  *slist;
    T*          object;
  };


  /// URBIStarter base class used to store heterogeneous template
  /// class objects in starterlist
  class baseURBIStarterHub
  {
  public:

    baseURBIStarterHub(const std::string& name) : name(name) {};
    virtual ~baseURBIStarterHub() {};

    /// Used to provide a wrapper to initialize objects in starterlist.
    virtual void init(const std::string&) = 0;
    virtual UObjectHub* getUObjectHub() = 0;
    std::string name;
  };

  //! This is the class containing URBI starters
  /** A starter is a class whose job is to start an instance of a particular UObject subclass,
   * resulting in the initialization of this object (registration to the kernel)
   */
  template <class T> class URBIStarterHub : public baseURBIStarterHub
  {
  public:
    URBIStarterHub(const std::string& name, UStartlistHub& _slist)
      : baseURBIStarterHub(name)
    {
      slist = &_slist;
      slist->push_back(dynamic_cast<baseURBIStarterHub*>(this));
    }
    virtual ~URBIStarterHub() {/* noone can kill a hub*/ };

  protected:
    /// Called when the object is ready to be initialized.
    virtual void init(const std::string& objname)
    {
      object = new T(objname);
    }

    /// Access to the object from the outside.
    virtual UObjectHub* getUObjectHub()
    {
      return dynamic_cast<UObjectHub*>(object);
    }

    UStartlistHub  *slist;
    T*              object;
  };


  template<typename T>
  class utrait
  {
  public:
    typedef T noref;
  };

# ifndef _MSC_VER

  template<typename T>
  class utrait<T&>
  {
  public:
    typedef T noref;
  };

# else

  template<>
  class utrait<UVar&>
  {
  public:
    typedef UVar noref;
  };

# endif

  /*-----------------------------------------------------.
  | This section is autogenerated. Not for humans eyes.  |
  `-----------------------------------------------------*/


       // non void return type

template <class OBJ, class R>
class UCallback0 : public UGenericCallback
{
 public:
  UCallback0(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) (),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 0, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)());
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ();
};

// void return type

template <class OBJ>
class UCallbackvoid0 : public UGenericCallback
{
 public:
  UCallbackvoid0(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) (),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,0, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)();
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void return type : special case for notifyend event callbacks

template <class OBJ>
class UCallbacknotifyend0 : public UGenericCallback
{
public:
  UCallbacknotifyend0(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) (),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,0, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU >
class UFCallbackvoid0 : public UGenericCallback
{
 public:
  UFCallbackvoid0(const std::string& objname, const std::string& type,
		    void (*fun) (),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,0, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)();
    return UValue();
  }

 private:
  void (*fun) ();
};

// non void non-member function
template <class R>
class UFCallback0 : public UGenericCallback
{
 public:
  UFCallback0(const std::string& objname, const std::string& type,
		R (*fun) (),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,0, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)());
  }

 private:
  R (*fun) ();
};

// callback creation for non-member non void return type

template <class R>
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) (),
		const std::string& funname, UTable& t)
{
  return new UFCallback0<R> (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template <> inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) (),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid0<void > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R>
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) (),
		const std::string& funname,UTable &t)
{
  return new UCallback0<OBJ,R> (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ>
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) (),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid0<OBJ> (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ>
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) (),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend0<OBJ> (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 >
class UCallback1 : public UGenericCallback
{
 public:
  UCallback1(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 1, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 );
};

// void return type

template <class OBJ, class P1 >
class UCallbackvoid1 : public UGenericCallback
{
 public:
  UCallbackvoid1(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,1, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 >
class UCallbacknotifyend1 : public UGenericCallback
{
public:
  UCallbacknotifyend1(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,1, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 >
class UFCallbackvoid1 : public UGenericCallback
{
 public:
  UFCallbackvoid1(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,1, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 );
};

// non void non-member function
template <class R, class P1 >
class UFCallback1 : public UGenericCallback
{
 public:
  UFCallback1(const std::string& objname, const std::string& type,
		R (*fun) ( P1 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,1, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 );
};

// callback creation for non-member non void return type

template <class R, class P1 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback1<R, P1 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid1<void , P1 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 ),
		const std::string& funname,UTable &t)
{
  return new UCallback1<OBJ,R, P1 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid1<OBJ, P1 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend1<OBJ, P1 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 >
class UCallback2 : public UGenericCallback
{
 public:
  UCallback2(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 2, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 );
};

// void return type

template <class OBJ, class P1 , class P2 >
class UCallbackvoid2 : public UGenericCallback
{
 public:
  UCallbackvoid2(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,2, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 >
class UCallbacknotifyend2 : public UGenericCallback
{
public:
  UCallbacknotifyend2(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,2, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 >
class UFCallbackvoid2 : public UGenericCallback
{
 public:
  UFCallbackvoid2(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,2, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 );
};

// non void non-member function
template <class R, class P1 , class P2 >
class UFCallback2 : public UGenericCallback
{
 public:
  UFCallback2(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,2, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback2<R, P1 , P2 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid2<void , P1 , P2 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 ),
		const std::string& funname,UTable &t)
{
  return new UCallback2<OBJ,R, P1 , P2 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid2<OBJ, P1 , P2 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend2<OBJ, P1 , P2 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 >
class UCallback3 : public UGenericCallback
{
 public:
  UCallback3(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 3, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 >
class UCallbackvoid3 : public UGenericCallback
{
 public:
  UCallbackvoid3(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,3, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 >
class UCallbacknotifyend3 : public UGenericCallback
{
public:
  UCallbacknotifyend3(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,3, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 >
class UFCallbackvoid3 : public UGenericCallback
{
 public:
  UFCallbackvoid3(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,3, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 >
class UFCallback3 : public UGenericCallback
{
 public:
  UFCallback3(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,3, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback3<R, P1 , P2 , P3 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid3<void , P1 , P2 , P3 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 ),
		const std::string& funname,UTable &t)
{
  return new UCallback3<OBJ,R, P1 , P2 , P3 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid3<OBJ, P1 , P2 , P3 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend3<OBJ, P1 , P2 , P3 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 >
class UCallback4 : public UGenericCallback
{
 public:
  UCallback4(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 4, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 >
class UCallbackvoid4 : public UGenericCallback
{
 public:
  UCallbackvoid4(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,4, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 >
class UCallbacknotifyend4 : public UGenericCallback
{
public:
  UCallbacknotifyend4(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,4, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 >
class UFCallbackvoid4 : public UGenericCallback
{
 public:
  UFCallbackvoid4(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,4, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 >
class UFCallback4 : public UGenericCallback
{
 public:
  UFCallback4(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,4, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback4<R, P1 , P2 , P3 , P4 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid4<void , P1 , P2 , P3 , P4 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
		const std::string& funname,UTable &t)
{
  return new UCallback4<OBJ,R, P1 , P2 , P3 , P4 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid4<OBJ, P1 , P2 , P3 , P4 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend4<OBJ, P1 , P2 , P3 , P4 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 >
class UCallback5 : public UGenericCallback
{
 public:
  UCallback5(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 5, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 >
class UCallbackvoid5 : public UGenericCallback
{
 public:
  UCallbackvoid5(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,5, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 >
class UCallbacknotifyend5 : public UGenericCallback
{
public:
  UCallbacknotifyend5(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,5, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 >
class UFCallbackvoid5 : public UGenericCallback
{
 public:
  UFCallbackvoid5(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,5, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 >
class UFCallback5 : public UGenericCallback
{
 public:
  UFCallback5(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,5, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback5<R, P1 , P2 , P3 , P4 , P5 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid5<void , P1 , P2 , P3 , P4 , P5 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
		const std::string& funname,UTable &t)
{
  return new UCallback5<OBJ,R, P1 , P2 , P3 , P4 , P5 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid5<OBJ, P1 , P2 , P3 , P4 , P5 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend5<OBJ, P1 , P2 , P3 , P4 , P5 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
class UCallback6 : public UGenericCallback
{
 public:
  UCallback6(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 6, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
class UCallbackvoid6 : public UGenericCallback
{
 public:
  UCallbackvoid6(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,6, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
class UCallbacknotifyend6 : public UGenericCallback
{
public:
  UCallbacknotifyend6(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,6, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
class UFCallbackvoid6 : public UGenericCallback
{
 public:
  UFCallbackvoid6(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,6, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
class UFCallback6 : public UGenericCallback
{
 public:
  UFCallback6(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,6, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback6<R, P1 , P2 , P3 , P4 , P5 , P6 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid6<void , P1 , P2 , P3 , P4 , P5 , P6 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		const std::string& funname,UTable &t)
{
  return new UCallback6<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid6<OBJ, P1 , P2 , P3 , P4 , P5 , P6 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend6<OBJ, P1 , P2 , P3 , P4 , P5 , P6 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
class UCallback7 : public UGenericCallback
{
 public:
  UCallback7(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 7, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
class UCallbackvoid7 : public UGenericCallback
{
 public:
  UCallbackvoid7(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,7, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
class UCallbacknotifyend7 : public UGenericCallback
{
public:
  UCallbacknotifyend7(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,7, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
class UFCallbackvoid7 : public UGenericCallback
{
 public:
  UFCallbackvoid7(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,7, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
class UFCallback7 : public UGenericCallback
{
 public:
  UFCallback7(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,7, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback7<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid7<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		const std::string& funname,UTable &t)
{
  return new UCallback7<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid7<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend7<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
class UCallback8 : public UGenericCallback
{
 public:
  UCallback8(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 8, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
class UCallbackvoid8 : public UGenericCallback
{
 public:
  UCallbackvoid8(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,8, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
class UCallbacknotifyend8 : public UGenericCallback
{
public:
  UCallbacknotifyend8(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,8, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
class UFCallbackvoid8 : public UGenericCallback
{
 public:
  UFCallbackvoid8(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,8, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
class UFCallback8 : public UGenericCallback
{
 public:
  UFCallback8(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,8, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback8<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid8<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		const std::string& funname,UTable &t)
{
  return new UCallback8<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid8<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend8<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
class UCallback9 : public UGenericCallback
{
 public:
  UCallback9(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 9, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
class UCallbackvoid9 : public UGenericCallback
{
 public:
  UCallbackvoid9(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,9, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
class UCallbacknotifyend9 : public UGenericCallback
{
public:
  UCallbacknotifyend9(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,9, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
class UFCallbackvoid9 : public UGenericCallback
{
 public:
  UFCallbackvoid9(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,9, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
class UFCallback9 : public UGenericCallback
{
 public:
  UFCallback9(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,9, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback9<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid9<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		const std::string& funname,UTable &t)
{
  return new UCallback9<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid9<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend9<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
class UCallback10 : public UGenericCallback
{
 public:
  UCallback10(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 10, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
class UCallbackvoid10 : public UGenericCallback
{
 public:
  UCallbackvoid10(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,10, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
class UCallbacknotifyend10 : public UGenericCallback
{
public:
  UCallbacknotifyend10(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,10, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
class UFCallbackvoid10 : public UGenericCallback
{
 public:
  UFCallbackvoid10(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,10, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
class UFCallback10 : public UGenericCallback
{
 public:
  UFCallback10(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,10, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback10<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid10<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		const std::string& funname,UTable &t)
{
  return new UCallback10<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid10<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend10<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
class UCallback11 : public UGenericCallback
{
 public:
  UCallback11(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 11, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
class UCallbackvoid11 : public UGenericCallback
{
 public:
  UCallbackvoid11(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,11, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
class UCallbacknotifyend11 : public UGenericCallback
{
public:
  UCallbacknotifyend11(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,11, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
class UFCallbackvoid11 : public UGenericCallback
{
 public:
  UFCallbackvoid11(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,11, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
class UFCallback11 : public UGenericCallback
{
 public:
  UFCallback11(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,11, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback11<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid11<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		const std::string& funname,UTable &t)
{
  return new UCallback11<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid11<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend11<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
class UCallback12 : public UGenericCallback
{
 public:
  UCallback12(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 12, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) , cast(param[12 - 1], (typename utrait<P12>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
class UCallbackvoid12 : public UGenericCallback
{
 public:
  UCallbackvoid12(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,12, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
class UCallbacknotifyend12 : public UGenericCallback
{
public:
  UCallbacknotifyend12(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,12, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
class UFCallbackvoid12 : public UGenericCallback
{
 public:
  UFCallbackvoid12(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,12, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
class UFCallback12 : public UGenericCallback
{
 public:
  UFCallback12(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,12, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback12<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid12<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		const std::string& funname,UTable &t)
{
  return new UCallback12<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid12<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend12<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
class UCallback13 : public UGenericCallback
{
 public:
  UCallback13(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 13, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) , cast(param[12 - 1], (typename utrait<P12>::noref*)0) , cast(param[13 - 1], (typename utrait<P13>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
class UCallbackvoid13 : public UGenericCallback
{
 public:
  UCallbackvoid13(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,13, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
class UCallbacknotifyend13 : public UGenericCallback
{
public:
  UCallbacknotifyend13(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,13, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
class UFCallbackvoid13 : public UGenericCallback
{
 public:
  UFCallbackvoid13(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,13, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
class UFCallback13 : public UGenericCallback
{
 public:
  UFCallback13(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,13, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback13<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid13<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		const std::string& funname,UTable &t)
{
  return new UCallback13<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid13<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend13<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
class UCallback14 : public UGenericCallback
{
 public:
  UCallback14(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 14, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) , cast(param[12 - 1], (typename utrait<P12>::noref*)0) , cast(param[13 - 1], (typename utrait<P13>::noref*)0) , cast(param[14 - 1], (typename utrait<P14>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
class UCallbackvoid14 : public UGenericCallback
{
 public:
  UCallbackvoid14(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,14, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
class UCallbacknotifyend14 : public UGenericCallback
{
public:
  UCallbacknotifyend14(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,14, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
class UFCallbackvoid14 : public UGenericCallback
{
 public:
  UFCallbackvoid14(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,14, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
class UFCallback14 : public UGenericCallback
{
 public:
  UFCallback14(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,14, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback14<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid14<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		const std::string& funname,UTable &t)
{
  return new UCallback14<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid14<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend14<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
class UCallback15 : public UGenericCallback
{
 public:
  UCallback15(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 15, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) , cast(param[12 - 1], (typename utrait<P12>::noref*)0) , cast(param[13 - 1], (typename utrait<P13>::noref*)0) , cast(param[14 - 1], (typename utrait<P14>::noref*)0) , cast(param[15 - 1], (typename utrait<P15>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
class UCallbackvoid15 : public UGenericCallback
{
 public:
  UCallbackvoid15(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,15, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
class UCallbacknotifyend15 : public UGenericCallback
{
public:
  UCallbacknotifyend15(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,15, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
class UFCallbackvoid15 : public UGenericCallback
{
 public:
  UFCallbackvoid15(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,15, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
class UFCallback15 : public UGenericCallback
{
 public:
  UFCallback15(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,15, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback15<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid15<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		const std::string& funname,UTable &t)
{
  return new UCallback15<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid15<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend15<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 > (objname, type,obj,fun,end,funname,t);
}


       // non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
class UCallback16 : public UGenericCallback
{
 public:
  UCallback16(const std::string& objname, const std::string& type,
	       OBJ* obj,
	       R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
	       const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname, 16, t),
      obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue(((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref*)0) , cast(param[2 - 1], (typename utrait<P2>::noref*)0) , cast(param[3 - 1], (typename utrait<P3>::noref*)0) , cast(param[4 - 1], (typename utrait<P4>::noref*)0) , cast(param[5 - 1], (typename utrait<P5>::noref*)0) , cast(param[6 - 1], (typename utrait<P6>::noref*)0) , cast(param[7 - 1], (typename utrait<P7>::noref*)0) , cast(param[8 - 1], (typename utrait<P8>::noref*)0) , cast(param[9 - 1], (typename utrait<P9>::noref*)0) , cast(param[10 - 1], (typename utrait<P10>::noref*)0) , cast(param[11 - 1], (typename utrait<P11>::noref*)0) , cast(param[12 - 1], (typename utrait<P12>::noref*)0) , cast(param[13 - 1], (typename utrait<P13>::noref*)0) , cast(param[14 - 1], (typename utrait<P14>::noref*)0) , cast(param[15 - 1], (typename utrait<P15>::noref*)0) , cast(param[16 - 1], (typename utrait<P16>::noref*)0) ));
  }

 private:
  OBJ* obj;
  R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 );
};

// void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
class UCallbackvoid16 : public UGenericCallback
{
 public:
  UCallbackvoid16(const std::string& objname, const std::string& type,
		   OBJ* obj,
		   void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		   const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,16, t),
    obj(obj), fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    ((*obj).*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) , cast(param[16 - 1], (typename utrait<P16>::noref *)0) );
    return UValue();
  }
 private:
  OBJ* obj;
  void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 );
};

// void return type : special case for notifyend event callbacks

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
class UCallbacknotifyend16 : public UGenericCallback
{
public:
  UCallbacknotifyend16(const std::string& objname, const std::string& type,
			OBJ* obj,
			void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
			void (OBJ::*end)(),const std::string& funname,
			UTable &t)
    : UGenericCallback(objname, type, funname,16, t),
    obj(obj), fun(end)
  {
  }

  virtual UValue __evalcall(UList &)
  {
    ((*obj).*fun)();
    return UValue();
  }

private:
  OBJ* obj;
  void (OBJ::*fun) ();
};

// void non-member function
template <class INU , class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
class UFCallbackvoid16 : public UGenericCallback
{
 public:
  UFCallbackvoid16(const std::string& objname, const std::string& type,
		    void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		    const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,16, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList &param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    (*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) , cast(param[16 - 1], (typename utrait<P16>::noref *)0) );
    return UValue();
  }

 private:
  void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 );
};

// non void non-member function
template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
class UFCallback16 : public UGenericCallback
{
 public:
  UFCallback16(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		const std::string& funname, UTable &t)
    : UGenericCallback(objname, type, funname,16, t),  fun(fun)
  {
  }

  virtual UValue __evalcall(UList& param)
  {
    // Avoid warnings when we don't use param.
    (void) param;
    return UValue((*fun)( cast(param[1 - 1], (typename utrait<P1>::noref *)0) , cast(param[2 - 1], (typename utrait<P2>::noref *)0) , cast(param[3 - 1], (typename utrait<P3>::noref *)0) , cast(param[4 - 1], (typename utrait<P4>::noref *)0) , cast(param[5 - 1], (typename utrait<P5>::noref *)0) , cast(param[6 - 1], (typename utrait<P6>::noref *)0) , cast(param[7 - 1], (typename utrait<P7>::noref *)0) , cast(param[8 - 1], (typename utrait<P8>::noref *)0) , cast(param[9 - 1], (typename utrait<P9>::noref *)0) , cast(param[10 - 1], (typename utrait<P10>::noref *)0) , cast(param[11 - 1], (typename utrait<P11>::noref *)0) , cast(param[12 - 1], (typename utrait<P12>::noref *)0) , cast(param[13 - 1], (typename utrait<P13>::noref *)0) , cast(param[14 - 1], (typename utrait<P14>::noref *)0) , cast(param[15 - 1], (typename utrait<P15>::noref *)0) , cast(param[16 - 1], (typename utrait<P16>::noref *)0) ));
  }

 private:
  R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 );
};

// callback creation for non-member non void return type

template <class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		R (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		const std::string& funname, UTable& t)
{
  return new UFCallback16<R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 > (objname, type,fun,funname,t);
}

// callback creation for non-member void return type
// inline required since one of them is not a template
template < class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 > inline
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		void (*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		const std::string& funname,UTable &t)
{
  return new UFCallbackvoid16<void , P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 > (objname, type,fun,funname,t);
}

// callback creation for non void return type

template <class OBJ, class R, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		R (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		const std::string& funname,UTable &t)
{
  return new UCallback16<OBJ,R, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 > (objname, type,obj,fun,funname,t);
}

// callback creation for void return type

template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		const std::string& funname,UTable &t)
{
  return new UCallbackvoid16<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 > (objname, type,obj,fun,funname,t);
}

// special case for eventend notification
template <class OBJ, class P1 , class P2 , class P3 , class P4 , class P5 , class P6 , class P7 , class P8 , class P9 , class P10 , class P11 , class P12 , class P13 , class P14 , class P15 , class P16 >
UGenericCallback*
createUCallback(const std::string& objname, const std::string& type,
		OBJ* obj,
		void (OBJ::*fun) ( P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 ),
		void (OBJ::*end)(), const std::string& funname,UTable &t)
{
  return new UCallbacknotifyend16<OBJ, P1 , P2 , P3 , P4 , P5 , P6 , P7 , P8 , P9 , P10 , P11 , P12 , P13 , P14 , P15 , P16 > (objname, type,obj,fun,end,funname,t);
}


} // end namespace urbi

#endif // ! URBI_UOBJECT_HH

/// Local Variables:
/// mode: c++
/// End:
