/*! \file uvalue-common.cc
 *******************************************************************************

 File: uvalue-common.cc\n
 Implementation of the UValue class and other linked classes

 This file is part of LIBURBI\n
 (c) Jean-Christophe Baillie, 2004-2006.

 Permission to use, copy, modify, and redistribute this software for
 non-commercial use is hereby granted.

 This software is provided "as is" without warranty of any kind,
 either expressed or implied, including but not limited to the
 implied warranties of fitness for a particular purpose.

 For more information, comments, bug reports: http://www.urbiforge.com

 **************************************************************************** */

#include "libport/cstdio"

#include <sstream>
#include <iostream>

#include "urbi/uobject.hh"

namespace urbi
{

  /*-----------------.
  | UValue Parsing.  |
  `-----------------*/

  void unescape(std::string& data)
  {
    int src = 0;
    int dst = 0;
    while (data[src])
    {
      if (data[src] != '\\')
	data[dst] = data[src];
      else
	switch(data[++src])
	{
	  case 'n':
	    data[dst] = '\n';
	    break;
	  case '\\':
	    data[dst] = '\\';
	    break;
	  case '"':
	    data[dst] = '"';
	    break;
	  default:
	    data[dst] = data[src];
	    break;
	}

      ++src;
      ++dst;
    }
    data[dst] = 0;
  }

  void unescape(char* data)
  {
    char* src = data;
    char* dst = data;
    while (*src)
    {
      if (*src != '\\')
	*dst = *src;
      else
	switch (*(++src))
	{
	  case 'n':
	    *dst = '\n';
	    break;
	  case '\\':
	    *dst = '\\';
	    break;
	  case '"':
	    *dst = '"';
	    break;
	  default:
	    *dst = *src;
	}

      ++src;
      ++dst;
    }
    *dst = 0;
  }

  /*---------.
  | UValue.  |
  `---------*/
  int UValue::parse(const char* message, int pos,
		    std::list<BinaryData>& bins,
		    std::list<BinaryData>::iterator& binpos)
  {
    while (message[pos] == ' ')
      ++pos;
    if (message[pos] == '"')
    {
      //string
      type = DATA_STRING;
      //get terminating '"'
      int p = pos + 1;
      while (message[p] && message[p] != '"')
      {
	if (message[p] == '\\')
	  ++p;
	++p;
      }
      if (!message[p])
	return -p; //parse error

      stringValue = new std::string(message + pos + 1, p - pos - 1);
      unescape(*stringValue);
      return p + 1;
    }

    if (message[pos] == '[')
    {
      //list message
      type = DATA_LIST;
      list = new UList();
      ++pos;
      while (message[pos] == ' ')
	++pos;
      while (message[pos])
      {
	while (message[pos] == ' ')
	  ++pos;
	if (message[pos] == ']')
	  break;
	UValue* v = new UValue();
	int p = v->parse(message, pos, bins, binpos);
	if (p < 0)
	  return p;
	list->array.push_back(v);
	pos = p;
	while (message[pos] == ' ')
	  ++pos;
	//expect , or rbracket
	if (message[pos] == ']')
	  break;
	if (message[pos] != ',')
	  return -pos;
	++pos;
      }

      if (message[pos] != ']')
	return -pos;
      return pos + 1;
    }

    //OBJ a [x:12, y:4]
    if (!strncmp(message + pos, "OBJ ", 4))
    {
      //obj message
      pos += 4;
      type = DATA_OBJECT;
      object = new UObjectStruct();

      while (message[pos] == ' ')
	++pos;
      if (message[pos] != '[')
	return -pos;
      ++pos;

      while (message[pos])
      {
	while (message[pos] == ' ')
	  ++pos;
	if (message[pos] == ']')
	  break; //empty object
	//parse name
	int p = pos;
	while (message[p] && message[p] != ':')
	  ++p;
	if (!message[p])
	  return -p; //parse error
	++p;
	UNamedValue nv;
	nv.name = std::string(message + pos, p - pos - 1);
	pos = p;
	while (message[pos] == ' ')
	  ++pos;
	UValue* v = new UValue();
	p = v->parse(message, pos, bins, binpos);
	if (p < 0)
	  return p;
	nv.val = v;
	object->array.push_back(nv);
	pos = p;
	while (message[pos] == ' ')
	  ++pos;
	//expect , or rbracket
	if (message[pos] == ']')
	  break;
	if (message[pos] != ',')
	  return -pos;
	++pos;
      }

      if (message[pos] != ']')
	return -pos;
      return pos + 1;
    }

    if (!strncmp(message+pos, "void", 4))
    {
      //void
      type = DATA_VOID;
      pos += 4;
      return pos;
    }

    if (!strncmp(message + pos, "BIN ", 4))
    {
      //binary message: delegate
      type = DATA_BINARY;
      binary = new UBinary();
      pos += 4;
      //parsing type
      int p = binary->parse(message, pos, bins, binpos);
      return p;
    }

    //last attempt: double
    int p;
    int count = sscanf(message+pos, "%lf%n", &val, &p);
    if (!count)
      return -pos;
    type = DATA_DOUBLE;
    pos += p;
    return pos;
  }

  std::ostream&
  UValue::print (std::ostream& s) const
  {
    switch (type)
    {
      case DATA_DOUBLE:
	s << val;
	break;
      case DATA_STRING:
	s << '"' << *stringValue << '"';
	break;
      case DATA_BINARY:
	if (binary->type != BINARY_NONE
	    && binary->type != BINARY_UNKNOWN)
	  binary->buildMessage();
	s << "BIN "<< binary->common.size << ' ' << binary->message << ';';
	s.write((char*) binary->common.data, binary->common.size);
	break;
      case DATA_LIST:
      {
	s << '[';
	int sz = list->size();
	for (int i = 0; i < sz; ++i)
	{
	  s << (*list)[i];
	  if (i != sz - 1)
	    s << " , ";
	}
	s << ']';
      }
      break;
      case DATA_OBJECT:
      {
	s << "OBJ "<<object->refName<<" [";
	int sz = object->size();
	for (int i = 0; i < sz; ++i)
	{
	  s << (*object)[i].name << ':';
	  s << (*object)[i].val;
	  if (i != sz - 1)
	    s << " , ";
	}
	s << ']';
      }
      break;
      default:
	s << "<<void>>";
    }
    return s;
  }


  /*---------.
  | USound.  |
  `---------*/

  const char* USound::format_string () const
  {
    switch (soundFormat)
    {
      case SOUND_RAW:
	return "raw";
      case SOUND_WAV:
	return "wav";
      case SOUND_MP3:
	return "mp3";
      case SOUND_OGG:
	return "ogg";
      case SOUND_UNKNOWN:
	return "unknown format";
    }
    // To pacify "warning: control reaches end of non-void function".
    abort();
  }

  /*---------.
  | UImage.  |
  `---------*/

  const char* UImage::format_string () const
  {
    switch (imageFormat)
    {
      case IMAGE_RGB:
	return "rgb";
      case IMAGE_JPEG:
	return "jpeg";
      case IMAGE_YCbCr:
	return "YCbCr";
      case IMAGE_PPM:
	return "ppm";
      case IMAGE_UNKNOWN:
	return "unknown format";
    }
    // To pacify "warning: control reaches end of non-void function".
    abort();
  }

  /*----------.
  | UBinary.  |
  `----------*/

  int
  UBinary::parse(const char* message, int pos,
		 std::list<BinaryData>& bins,
		 std::list<BinaryData>::iterator& binpos)
  {
    while (message[pos] == ' ')
      ++pos;
    //find end of header

    if (binpos == bins.end()) //no binary data available
      return -1;

    //validate size
    int ps;
    unsigned psize;
    int count = sscanf(message+pos, "%u%n", &psize, &ps);
    if (count != 1)
      return -pos;
    if (psize != binpos->size)
    {
      std::cerr << "bin size inconsistency\n";
      return -pos;
    }
    pos += ps;
    common.size = psize;
    common.data = malloc(psize);
    memcpy(common.data, binpos->data, common.size);
    ++binpos;

    int p = pos;
    while (message[p] && message[p] != '\n')
      ++p;
    if (!message[p])
      return -p; //parse error
    this->message = std::string(message + pos, p - pos);
    ++p;

    //trying to parse header to find type
    char type[64];
    memset(type, 0, 64);
    int p2, p3, p4, p5;
    count = sscanf(message + pos, "%63s %d %d %d %d",
		   type, &p2, &p3, &p4, &p5);
    //DEBUG fprintf(stderr, "%s:	 %d %s %d %d\n", message, p1, type, p2, p3);
    if (STREQ(type, "jpeg"))
    {
      this->type = BINARY_IMAGE;
      image.size = common.size;
      image.width = p2;
      image.height = p3;
      image.imageFormat = IMAGE_JPEG;
      return p;
    }

    if (STREQ(type, "YCbCr"))
    {
      this->type = BINARY_IMAGE;
      image.size = common.size;
      image.width = p2;
      image.height = p3;
      image.imageFormat = IMAGE_YCbCr;
      return p;
    }

    if (STREQ(type, "rgb"))
    {
      this->type = BINARY_IMAGE;
      image.size = common.size;
      image.width = p2;
      image.height = p3;
      image.imageFormat = IMAGE_RGB;
      return p;
    }

    if (STREQ(type, "raw"))
    {
      this->type = BINARY_SOUND;
      sound.soundFormat = SOUND_RAW;
      sound.size = common.size;
      sound.channels = p2;
      sound.rate = p3;
      sound.sampleSize = p4;
      sound.sampleFormat = (USoundSampleFormat) p5;
      return p;
    }

    if (STREQ(type, "wav"))
    {
      this->type = BINARY_SOUND;
      sound.soundFormat = SOUND_WAV;
      sound.size = common.size;
      sound.channels = p2;
      sound.rate = p3;
      sound.sampleSize = p4;
      sound.sampleFormat = (USoundSampleFormat) p5;
      return p;
    }

    //unknown binary
    this->type = BINARY_UNKNOWN;
    return p;
  }

  void UBinary::buildMessage()
  {
    message = getMessage();
  }

  std::string UBinary::getMessage() const
  {
    std::ostringstream o;
    switch (type)
    {
      case BINARY_IMAGE:
	o << image.format_string()
	  << ' ' << image.width << ' ' << image.height;
	break;

      case BINARY_SOUND:
	o << sound.format_string()
	  << ' ' << sound.channels
	  << ' ' << sound.rate
	  << ' ' << sound.sampleSize
	  << ' ' << sound.sampleFormat;
	break;

      case BINARY_UNKNOWN:
	o << message;
	break;

      case BINARY_NONE:
	abort();
    }
    return o.str();
  }

  /*---------.
  | UValue.  |
  `---------*/

  //! Class UValue implementation
  UValue::UValue()
    : type(DATA_VOID), val (0), storage(0)
  {}


  UValue::UValue(ufloat v)
    : type(DATA_DOUBLE), val(v)
  {}

  UValue::UValue(int v)
    : type(DATA_DOUBLE), val(v)
  {}
  UValue::UValue(unsigned int v)
    : type(DATA_DOUBLE), val(v)
  {}
  UValue::UValue(long v)
    : type(DATA_DOUBLE), val(v)
  {}
  UValue::UValue(unsigned long v)
    : type(DATA_DOUBLE), val(v)
  {}

  UValue::UValue(void* v)
    : type(DATA_STRING)
  {
    std::ostringstream i;
    i << "%ptr_" << (unsigned long) v;
    stringValue = new std::string(i.str());
  }
  UValue::UValue(char* v)
    : type(DATA_STRING), stringValue(new std::string(v))
  {}

  UValue::UValue(const std::string& v)
    : type(DATA_STRING), stringValue(new std::string(v))
  {}

  UValue::UValue(const UBinary& b)
    : type(DATA_BINARY)
  {
    binary = new UBinary(b);
  }

  UValue::UValue(const USound& s)
    : type(DATA_BINARY)
  {
    binary = new UBinary(s);
  }

  UValue::UValue(const UImage& s)
    : type(DATA_BINARY)
  {
    binary = new UBinary(s);
  }

  UValue::UValue(const UList& l)
    : type(DATA_LIST)
  {
    list = new UList(l);
  }

  UValue::UValue(const UObjectStruct& o)
    : type(DATA_OBJECT)
  {
    object = new UObjectStruct(o);
  }

  UValue::~UValue()
  {
    switch(type)
    {
      case DATA_STRING:
	delete stringValue;
	break;
      case DATA_BINARY:
	delete binary;
	break;
      case DATA_LIST:
	delete list;
	break;
      case DATA_OBJECT:
	delete object;
	break;
      case DATA_DOUBLE:
      case DATA_VOID:
	break;
    }
  }

  UValue::operator ufloat () const
  {
    switch (type)
    {
      case DATA_DOUBLE:
	return val;

      case DATA_STRING:
      {
	std::istringstream tstr(*stringValue);
	ufloat v;
	tstr >> v;
	return v;
      }
      break;

      case DATA_BINARY:
      case DATA_LIST:
      case DATA_OBJECT:
      case DATA_VOID:
	break;
    }

    return ufloat(0);
  }


  UValue::operator std::string() const
  {
    switch (type)
    {
      case DATA_DOUBLE:
      {
	std::ostringstream tstr;
	tstr << val;
	return tstr.str();
      }
      break;
      case DATA_STRING:
	return *stringValue;
	break;
      default:
	return std::string("invalid");
    }
  }

  UValue::operator
  UBinary() const
  {
    if (type != DATA_BINARY)
      return UBinary();
    else
      return *binary;
  }

  UValue::operator UImage() const
  {
    if (type != DATA_BINARY
	|| binary->type != BINARY_IMAGE)
    {
      UImage i;
      i.data = 0;
      i.size = i.width = i.height = 0;
      i.imageFormat = IMAGE_UNKNOWN;
      return i;
    }
    else
      return binary->image;
  }

  UValue::operator USound() const
  {
    if (type != DATA_BINARY || binary->type != BINARY_SOUND)
    {
      USound i;
      i.data = 0;
      i.size = i.sampleSize = i.channels = i.rate = 0;
      i.soundFormat = SOUND_UNKNOWN;
      i.sampleFormat = SAMPLE_UNSIGNED;
      return i;
    }
    else
      return binary->sound;
  }

  UValue::operator UList() const
  {
    if (type != DATA_LIST)
      return UList();
    return UList(*list);
  }

  UValue& UValue::operator= (const UValue& v)
  {
    // TODO: optimize
    if (this == &v)
      return *this;
    switch (type)
    {
      case DATA_STRING:
	delete stringValue;
	break;
      case DATA_BINARY:
	delete binary;
	break;
      case DATA_LIST:
	delete list;
	break;
      case DATA_OBJECT:
	delete object;
      case DATA_DOUBLE:
      case DATA_VOID:
	break;
    }

    type = v.type;
    switch (type)
    {
      case DATA_DOUBLE:
	val = v.val;
	break;
      case DATA_STRING:
	stringValue = new std::string(*v.stringValue);
	break;
      case DATA_BINARY:
	binary = new UBinary(*v.binary);
	break;
      case DATA_LIST:
	list = new UList(*v.list);
	break;
      case DATA_OBJECT:
	object = new UObjectStruct(*v.object);
	break;
      case DATA_VOID:
	break;
    }
    return *this;
  }


  UValue::UValue(const UValue& v)
  {
    type = DATA_VOID;
    *this = v;
  }

  /*----------.
  | UBinary.  |
  `----------*/

  UBinary::UBinary()
  {
    common.data = 0;
    common.size = 0;
    type = BINARY_NONE;
  }

  UBinary::~UBinary()
  {
    if (common.data)
      free(common.data);
  }

  UBinary::UBinary(const UBinary& b)
  {
    type = BINARY_NONE;
    common.data = 0;
    *this = b;
  }

  UBinary::UBinary(const UImage& i)
  {
    type = BINARY_IMAGE;
    image = i;
    image.data = static_cast<unsigned char*> (malloc (image.size));
    memcpy(image.data, i.data, image.size);
  }

  UBinary::UBinary(const USound& i)
  {
    type = BINARY_SOUND;
    sound = i;
    sound.data = static_cast<char*> (malloc (sound.size));
    memcpy(sound.data, i.data, sound.size);
  }

  UBinary& UBinary::operator= (const UBinary& b)
  {
    if (this == &b)
      return *this;

    free(common.data);

    type = b.type;
    message = b.message;
    common.size = b.common.size;
    switch(type)
    {
      case BINARY_IMAGE:
	image = b.image;
	break;
      case BINARY_SOUND:
	sound = b.sound;
	break;
      case BINARY_NONE:
      case BINARY_UNKNOWN:
	break;
    }
    common.data = malloc(common.size);
    memcpy(common.data, b.common.data, b.common.size);
    return *this;
  }


  /*--------.
  | UList.  |
  `--------*/

  UList::UList()
    : offset(0)
  {}

  UList::UList(const UList& b)
    : offset(0)
  {
    *this = b;
  }

  UList& UList::operator= (const UList& b)
  {
    if (this == &b)
      return *this;

    offset = 0;
    for (int i = 0; i < size(); ++i) //relax, its a vector
      delete array[i];
    array.clear();

    for (std::vector<UValue*>::const_iterator it = b.array.begin();
	 it !=b.array.end();
	 ++it)
      array.push_back(new UValue(**it));
    offset = b.offset;

    return *this;
  }

  UList::~UList()
  {
    offset = 0;
    for (int i = 0; i < size(); ++i) //relax, its a vector
      delete array[i];
    array.clear();
  }


  /*----------------.
  | UObjectStruct.  |
  `----------------*/

  UObjectStruct::UObjectStruct()
  {}

  UObjectStruct::UObjectStruct(const UObjectStruct& b)
  {
    *this = b;
  }

  UObjectStruct& UObjectStruct::operator= (const UObjectStruct& b)
  {
    if (this == &b)
      return *this;

    for (int i = 0; i < size(); ++i) // Relax, it's a vector.
      delete array[i].val;
    array.clear();

    for (std::vector<UNamedValue>::const_iterator it = b.array.begin();
	 it != b.array.end();
	 ++it)
      array.push_back(UNamedValue(it->name, new UValue(*(it->val))));

    return *this;
  }

  UObjectStruct::~UObjectStruct()
  {
    for (int i = 0; i < size(); ++i) //relax, it's a vector
      delete array[i].val;
    array.clear();
  }


  /*---------.
  | UValue.  |
  `---------*/

  UValue& UObjectStruct::operator[] (const std::string& s)
  {
    for (int i = 0; i < size(); ++i)
      if (array[i].name == s)
	return *array[i].val;
    static UValue n;
    return n; // Gni?
  }

  /*-------.
  | UVar.  |
  `-------*/

  void
  UVar::operator= (const UValue& v)
  {
    switch (v.type)
    {
      case DATA_STRING:
	*this = *v.stringValue;
	break;
      case DATA_BINARY:
	*this = *v.binary;
	break;
      case DATA_LIST:
	*this = *v.list;
	break;
      case DATA_DOUBLE:
	*this = v.val;
	break;
      case DATA_VOID:
	//TODO: do something!
	abort ();
	break;
      case DATA_OBJECT:
	// Not valid currently.
	abort ();
	break;
    }
  }

} // namespace urbi
